"""Init."""
__version__ = "0.1.0a0"
from .deeplx_tr import deeplx_tr, lang_list
from .deeplx_client import deeplx_client
from .deeplx_client_async import deeplx_client_async

__all__ = ("deeplx_tr", "deeplx_client", "deeplx_client_async", "lang_list")



"""Test deeplx_tr."""

# pylint: disable=broad-except
import os
import time

from deeplx_tr import __version__, deeplx_tr
from loguru import logger

IDLE_TIME = 15
# e.g. set/export IDLE_TIME=10 to change IDLE_TIME time
try:
    _ = int(os.getenv(IDLE_TIME))
except Exception:
    _ = IDLE_TIME

IDLE_TIME = _ or IDLE_TIME


def test_version():
    """Test version."""
    assert __version__[:3] == "0.1"


def test_sanity():
    """Check sanity."""
    try:
        assert not deeplx_tr()
    except Exception:
        assert True


def test_deeplx_tr_whitespaces():
    """Test whitespaces stripped."""
    # _ = deeplx_tr({"text": "   "})
    _ = deeplx_tr(text="   ")
    assert _.get("data") == ""


def test_deeplx_tr_simple_default():
    """Test simple 'this is a test'."""
    # text = "This is a test."
    text = "This is test ."

    logger.info(f" sleeping for {IDLE_TIME} sec")
    time.sleep(IDLE_TIME)  # roug rate limit

    # _ = deeplx_tr({"text": text})  # {'code': 200, 'data': '这是一个 测试。', 'id': 101564000}
    _ = deeplx_tr(text=text)
    _ = _.get("data")

    assert "测" in _ or "试" in _ or "考" in _


def test_deeplx_tr_simple_tgt_lang_de():
    """Test simple 'this is a test'."""
    text = "This is a test."

    logger.info(f" sleeping for {IDLE_TIME} sec")
    time.sleep(IDLE_TIME)  # roug rate limit

    # _ = deeplx_tr({"text": text, "target_lang": "de"})
    _ = deeplx_tr(text=text, target_lang="de")
    _ = _.get("data").lower()  # 'dies ist ein test.'

    assert "dies" in _ or "test" in _



"""
Test deeplx_client.

Use -s or --capture=no e.g., pytest -s test_foobar.py to show output
"""
from deeplx_tr.deeplx_client import deeplx_client as cl
from loguru import logger


def test_deeplx_client_simple_sync():
    """Test deeplx_client_simple."""
    _ = cl("Test")
    logger.info(_)
    assert "测" in _

    _ = cl("Test", alternatives=1)
    logger.info(_)
    assert "试" in _



"""
Test async deeplx_client.

Use -s or --capture=no e.g., pytest -s test_foobar.py to show output
"""
import pytest
from deeplx_tr.deeplx_client_async import deeplx_client_async as cl
from loguru import logger

pytestmark = pytest.mark.asyncio

async def test_deeplx_client_async():
    """Test deeplx_client_async."""
    _ = await cl("Test")
    logger.info(_)
    assert "测" in _

    _ = await cl("Test", alternatives=1)
    logger.info(_)
    assert "试" in _



"""
Translate via deepl, based on deeplx, credits to 小芍同学 and chatgpt.

Supported languages: 'de', 'en', 'es', 'fr', 'it', 'ja', 'ko', 'nl',
  'pl', 'pt', 'ru', 'zh','bg', 'cs', 'da', 'el', 'et', 'fi', 'hu', 'lt',
  'lv', 'ro', 'sk', 'sl', 'sv'
"""
# pylint: disable=invalid-name, line-too-long
# pip install set-loglevel loguru icecream httpx pyperclip typer
# pip install nuitka ordered-set
import json
import os
import random

# import string
import sys
import time
from typing import List, Optional

# import gjson
# import requests
import httpx
import pyperclip
import typer

# import pygments  # for nuikta packaging
# from icecream import ic
from loguru import logger
from set_loglevel import set_loglevel

url = "https://www2.deepl.com/jsonrpc"

if os.environ.get("LOGURU_LEVEL") is None:
    logger.remove()
    logger.add(sys.stderr, level=set_loglevel())

_ = """
ic.configureOutput(
    includeContext=True,
    outputFunction=logger.debug,  # outputFunction=logger.info,
)
# """
__version__ = "0.0.1"
lang_list = [
    "de",
    "en",
    "es",
    "fr",
    "it",
    "ja",
    "ko",
    "nl",
    "pl",
    "pt",
    "ru",
    "zh",
    "bg",
    "cs",
    "da",
    "el",
    "et",
    "fi",
    "hu",
    "lt",
    "lv",
    "ro",
    "sk",
    "sl",
    "sv",
]

app = typer.Typer(
    name="deeplx-tr",
    add_completion=False,
    help=f"deeplx translate, supported languages: {lang_list}",
)


def _version_callback(value: bool) -> None:
    if value:
        typer.echo(f"{app.info.name} v.{__version__} -- {__doc__}")
        raise typer.Exit()


def init_data(source_lang: Optional[str], target_lang: Optional[str]):
    """Prep initial data."""
    return {
        "jsonrpc": "2.0",
        "method": "LMT_handle_texts",
        "id": random.randint(100000, 109999) * 1000,
        "params": {
            "splitting": "newlines",
            "lang": {
                "source_lang_user_selected": source_lang,
                "target_lang": target_lang,
            },
        },
    }


def get_i_count(translate_text: str) -> int:
    """Count i."""
    return translate_text.count("i")


def get_random_number() -> int:
    """Gen a random number."""
    return random.randint(100000, 109999) * 1000


def get_timestamp(i_count: int) -> int:
    """Gen timestamp."""
    ts = int(time.time() * 1000)
    if i_count != 0:
        i_count = i_count + 1
        return ts - ts % i_count + i_count

    # else:
    return ts


# def deeplx_tr(data: dict) -> dict:
def deeplx_tr(
    text: str,
    source_lang: str = "auto",
    target_lang: str = "zh",
) -> dict:
    """Translate."""
    # source_lang = data.get("source_lang")
    # target_lang = data.get("target_lang", "zh")

    if source_lang is not None:
        source_lang = source_lang.lower()
    if target_lang is not None:
        target_lang = target_lang.lower()

    # translate_text = data.get("text", "")
    translate_text = text[:]

    # handle whitespace input and non-string input
    try:
        translate_text = translate_text.strip()
    except Exception:
        translate_text = str(translate_text)

    if not translate_text:
        return {"data": ""}

    if source_lang not in lang_list:
        source_lang = None
    if target_lang not in lang_list:
        if source_lang not in ["zh"]:
            target_lang = "zh"
        else:
            target_lang = "en"

    post_data = init_data(source_lang, target_lang)
    text = {
        "text": translate_text,
        # "requestAlternatives": 3,
    }
    post_data["params"]["texts"] = [text]
    post_data["params"]["timestamp"] = get_timestamp(get_i_count(translate_text))
    post_str = json.dumps(post_data)
    if (post_data["id"] + 5) % 29 == 0 or (post_data["id"] + 3) % 13 == 0:
        post_str = post_str.replace('"method":"', '"method" : "', 1)
    else:
        post_str = post_str.replace('"method":"', '"method": "', 1)

    logger.debug(post_str)

    try:
        # response = requests.post(url, post_str, headers={'Content-Type': 'application/json'})
        response = httpx.post(
            url, data=post_str, headers={"Content-Type": "application/json"}  # type: ignore
        )

        # response = requests.post(url, post_data, headers={'Content-Type': 'application/json'})
        # print(response.text)
        # response_json = json.loads(response.text)
        response_json = response.json()
    except Exception as e:
        raise e

    if "error" in response_json:
        error = response_json["error"]
        raise Exception(error.get("message"))

    # else:
    return {
        "code": 200,
        "id": post_data["id"],
        "data": response_json["result"]["texts"][0]["text"],
    }


@app.command()
def main(
    version: Optional[bool] = typer.Option(  # pylint: disable=(unused-argument
        None,
        "--version",
        "-v",
        "-V",
        help="Show version info and a list of supported languages and exit.",
        callback=_version_callback,
        is_eager=True,
    ),
    text: Optional[List[str]] = typer.Argument(
        None,
        help="Source text.",
    ),
    clipb: Optional[bool] = typer.Option(
        None,
        "--clipb",
        "-c",
        help="Use clipboard content if set or if `text` is empty.",
    ),
    from_lang: Optional[str] = typer.Option(
        None, "--from-lang", "-f", help="Source language."
    ),
    to_lang: Optional[str] = typer.Option(
        "zh", "--to-lang", "-t", help="Target language."
    ),
):
    rf"""
    deeplx in plain python.

    supported languages: {lang_list}

    set LOGURU_LEVEL=DEBUG or set LOGLEVEL=10 to turn on debug/verbose mode.
    """
    logger.debug(["text", "clipb", "from_lang", "to_lang"])
    logger.debug(text)
    logger.debug(clipb)
    logger.debug(from_lang)
    logger.debug(to_lang)

    # ic(text)

    # collect text
    # check clipb first, then terminal
    # if nothing supplied from terminal, check clipb again
    if clipb:
        text_str = pyperclip.paste()
    else:
        if text is None:
            text_str = ""
        else:
            text_str = " ".join(text).strip()
        if not text_str:
            text_str = pyperclip.paste()

    logger.debug(text_str)
    # ic(text_str)

    if not text_str:
        _ = "Either supply some text from the terminal or set -c (--clipb) to translate clipboard content (make sure the clipboard is not empty)."
        msg = typer.style(_, fg=typer.colors.GREEN, bold=True)
        typer.echo(msg)
        _ = input("Press Enter to continue...")
        raise typer.Exit(code=1)

    data = {
        "text": text_str,
        "source_lang": from_lang,
        "target_lang": to_lang,
    }

    try:
        resp = deeplx_tr(**data)
    except Exception as e:
        logger.error(e)
        raise typer.Exit(code=1)

    logger.debug(resp)
    # ic(resp)

    _ = resp.get("data", "").strip()
    typer.echo(_)

    if _:
        pyperclip.copy(_)


if __name__ == "__main__":
    # uvicorn.run(app="main:app", host="127.0.0.1", port=8080, reload=True)
    # print(sys.argv[1:], bool(sys.argv[1:]), _)
    _ = """
    if sys.argv[1:]:
        _ = {"text": " ".join(sys.argv[1:])}
    else:
        _ = {"text": "this is a test"}

    try:
        res = deeplx_tr(_)
        print(_, res)
    except Exception as exc:
        print('exc: ', exc)
        logger.error(exc)
    # """
    app()



"""
Fetch translation from api.deeplx.org and variants.

async version of deeplx_client

Unrelated to deeplx_tr.py

curl -X POST https://api.deeplx.org/translate -d '{
    "text": "Hello, world!",
    "source_lang": "de",
    "target_lang": "zh"
}'

example return:
{"code":200,"id":865910002,"data":"你好，世界","alternatives":["世界，你好","你好，世界！","大家好"]}
"""
# pylint: disable=invalid-name,too-many-branches, too-many-statements
import asyncio
import os
import sys
from typing import Union

import httpx
from loguru import logger

deeplx_url = "https://api.deeplx.org/translate"
client = httpx.AsyncClient()

CONCURRENCY_LIMIT = 5
try:
    _ = str(CONCURRENCY_LIMIT)
    _ = int(os.getenv("CONCURRENCY_LIMIT", _))
    _ = max(_, 1)  # set to 1 if negative or 0
except (TypeError, ValueError) as exc:
    logger.trace(
        f"env var CONCURRENCY_LIMIT not set or is invalid: {exc}, "
        "setting CONCURRENCY_LIMIT to 5"
    )
    _ = CONCURRENCY_LIMIT
# override if env var os.getenv("CONCURRENCY_LIMIT" is properly set
CONCURRENCY_LIMIT = _

semaphore = asyncio.Semaphore(CONCURRENCY_LIMIT)


async def deeplx_client_async(
    text: str,
    source_lang: str = "",
    target_lang: str = "",
    alternatives: bool = False,
    url: Union[str, None] = None,
) -> str:
    """
    Translate via api.deeplx.org and variants.

    async version of deeplx_client

    Unrelated to deeplx_tr.py

    Args:
    ----
    text: to be translated
    source_lang: source language (default auto)
    target_lang: default zh (chinese)
    alternatives: also output available alternatives if set (default False)
    url: deeplx api url, default https://api.deeplx.org/translate

    Returns:
    -------
    translation

    """
    # url = deeplx_url
    # if os.getenv("DEEPLX_URL") is not None:
    if url is None:
        url = deeplx_url
    else:
        try:
            url = url.strip()
        except Exception:
            logger.warning("Unable to process {url=}, setting to ''.")
            url = ""

    url = url or os.getenv("DEEPLX_URL")

    assert url, f"{url=}, url must not be empty or None"

    try:
        text = str(text).strip()
    except Exception as exc:
        logger.error(exc)
        raise
    if not text:
        logger.warning("empty input, nothing to do, return ''.")
        return ""

    try:
        source_lang = source_lang.strip()
    except Exception:
        source_lang = ""
    try:
        target_lang = target_lang.strip()
    except Exception:
        target_lang = "zh"

    if source_lang.lower() in ["chinese", "zhong", "zhongwen"]:
        source_lang = "zh"
    if target_lang.lower() in ["chinese", "zhong", "zhongwen"]:
        target_lang = "zh"

    if source_lang.lower() in ["english", "eng"]:
        source_lang = "en"
    if target_lang.lower() in ["english", "eng"]:
        target_lang = "en"

    if source_lang.lower() in ["deutsch", "german", "ger"]:
        source_lang = "de"
    if target_lang.lower() in ["deutsch", "german", "ger"]:
        target_lang = "de"

    if source_lang in [""] and target_lang in [""]:
        target_lang = "zh"
    logger.trace(f"{source_lang=}, {target_lang=}")

    if source_lang in ["en"] and target_lang in [""]:
        source_lang = "zh"
    if source_lang in ["zh"] and target_lang in [""]:
        source_lang = "en"

    logger.trace(f"{text=}")
    logger.trace(f"{source_lang=}, {target_lang=}")

    if source_lang == target_lang:
        return text

    data = dict(
        text=text,
        source_lang=source_lang,
        target_lang=target_lang,
    )

    logger.trace(f"{data=}")

    async with semaphore:
        try:
            resp = await client.post(url, json=data)  # type: ignore
            resp.raise_for_status()
        except Exception as exc:
            logger.error(exc)
            raise

    try:
        res = resp.json().get("data")
        alt_output = resp.json().get("alternatives")
    except Exception as exc:
        logger.error(exc)
        raise

    if alternatives and alt_output:
        return f"{res} / {', '.join(alt_output)}"

    return res


async def main():
    """Testrun, nothing fancy."""
    text = " ".join(sys.argv[1:])
    if not text:
        text = "test"

    print(f"{text=}")
    print(
        await asyncio.gather(
            deeplx_client_async(text),
            deeplx_client_async("hello world"),
            deeplx_client_async(text, alternatives=True),
            deeplx_client_async("hello world", alternatives=True),
        )
    )


if __name__ == "__main__":
    asyncio.run(main())



"""Prep __main__.py."""
# pylint: disable=invalid-name
import os
import sys
from typing import List, Optional

import pyperclip
import typer
from loguru import logger
from set_loglevel import set_loglevel

from deeplx_tr import __version__, deeplx_tr, lang_list

if os.environ.get("LOGURU_LEVEL") is None:
    logger.remove()
    logger.add(sys.stderr, level=set_loglevel())


del sys
# logger.remove()
# logger.add(sys.stderr, level="TRACE")


app = typer.Typer(
    name="deeplx-tr",
    add_completion=False,
    help="deeplx-tr help",
)


def _version_callback(value: bool) -> None:
    if value:
        typer.echo(f"{app.info.name} v.{__version__} -- supported languages: {lang_list}")
        raise typer.Exit()


@app.command()
def main(
    version: Optional[bool] = typer.Option(  # pylint: disable=(unused-argument
        None,
        "--version",
        "-v",
        "-V",
        help="Show version info and a list of supported languages and exit.",
        callback=_version_callback,
        is_eager=True,
    ),
    text: Optional[List[str]] = typer.Argument(
        None,
        help="Source text.",
    ),
    clipb: Optional[bool] = typer.Option(
        None,
        "--clipb",
        "-c",
        help="Use clipboard content if set or if `text` is empty.",
    ),
    from_lang: Optional[str] = typer.Option(
        None, "--from-lang", "-f", help="Source language."
    ),
    to_lang: Optional[str] = typer.Option(
        "zh", "--to-lang", "-t", help="Target language."
    ),
):
    rf"""
    deeplx translate in plain python.

    supported languages: {lang_list}

    set LOGURU_LEVEL=DEBUG or set LOGLEVEL=10 to turn on debug/verbose mode.
    """
    logger.debug(["text", "clipb", "from_lang", "to_lang"])
    logger.debug(text)
    logger.debug(clipb)
    logger.debug(from_lang)
    logger.debug(to_lang)

    # ic(text)

    # collect text
    # check clipb first, then terminal
    # if nothing supplied from terminal, check clipb again
    if clipb:
        text_str = pyperclip.paste()
    else:
        if text is None:
            text_str = ""
        else:
            text_str = " ".join(text).strip()
        if not text_str:
            text_str = pyperclip.paste()

    logger.debug(text_str)
    # ic(text_str)

    if not text_str:
        _ = "Either supply some text from the terminal or set -c (--clipb) to translate clipboard content (make sure the clipboard is not empty)."
        msg = typer.style(_, fg=typer.colors.GREEN, bold=True)
        typer.echo(msg)
        _ = input("Press Enter to continue...")
        raise typer.Exit(code=1)

    data = {
        "text": text_str,
        "source_lang": from_lang,
        "target_lang": to_lang,
    }

    try:
        resp = deeplx_tr(**data)
    except Exception as e:
        logger.error(e)
        raise typer.Exit(code=1)

    logger.debug(resp)
    # ic(resp)

    _ = resp.get("data", "").strip()
    typer.echo(_)

    if _:
        pyperclip.copy(_)


if __name__ == "__main__":
    app()
